@@ -42,8 +42,9 @@
 class QgsMapToPixel;
 class QgsRasterRenderer;
 class QgsRectangle;
+class QgsRasterLayerTemporalProperties;
+
 class QImage;
-class QLibrary;
 class QPixmap;
 class QSlider;
 
@@ -51,90 +52,21 @@ typedef QList < QPair< QString, QColor > > QgsLegendColorList;
 
 /**
  * \ingroup core
- *  This class provides qgis with the ability to render raster datasets
- *  onto the mapcanvas.
  *
- *  The qgsrasterlayer class makes use of gdal for data io, and thus supports
- *  any gdal supported format. The constructor attempts to infer what type of
- *  file (LayerType) is being opened - not in terms of the file format (tif, ascii grid etc.)
- *  but rather in terms of whether the image is a GRAYSCALE, PaletteD or Multiband,
+ * Represents a raster layer.
  *
- *  Within the three allowable raster layer types, there are 8 permutations of
- *  how a layer can actually be rendered. These are defined in the DrawingStyle enum
- *  and consist of:
- *
- *  SingleBandGray -> a GRAYSCALE layer drawn as a range of gray colors (0-255)
- *  SingleBandPseudoColor -> a GRAYSCALE layer drawn using a pseudocolor algorithm
- *  PalettedSingleBandGray -> a PaletteD layer drawn in gray scale (using only one of the color components)
- *  PalettedSingleBandPseudoColor -> a PaletteD layer having only one of its color components rendered as pseudo color
- *  PalettedMultiBandColor -> a PaletteD image where the bands contains 24bit color info and 8 bits is pulled out per color
- *  MultiBandSingleBandGray -> a layer containing 2 or more bands, but using only one band to produce a grayscale image
- *  MultiBandSingleBandPseudoColor -> a layer containing 2 or more bands, but using only one band to produce a pseudocolor image
- *  MultiBandColor -> a layer containing 2 or more bands, mapped to the three RGBcolors. In the case of a multiband with only two bands, one band will have to be mapped to more than one color
- *
- *  Each of the above mentioned drawing styles is implemented in its own draw* function.
- *  Some of the drawing styles listed above require statistics about the layer such
- *  as the min / max / mean / stddev etc. statistics for a band can be gathered using the
- *  bandStatistics function. Note that statistics gathering is a slow process and
- *  every effort should be made to call this function as few times as possible. For this
- *  reason, qgsraster has a vector class member to store stats for each band. The
- *  constructor initializes this vector on startup, but only populates the band name and
- *  number fields.
- *
- *  Note that where bands are of gdal 'undefined' type, their values may exceed the
- *  renderable range of 0-255. Because of this a linear scaling histogram enhanceContrast is
- *  applied to undefined layers to normalise the data into the 0-255 range.
- *
- *  A qgsrasterlayer band can be referred to either by name or by number (base=1). It
- *  should be noted that band names as stored in datafiles may not be unique, and
- *  so the rasterlayer class appends the band number in brackets behind each band name.
+ * A QgsRasterLayer is instantiated by specifying the name of a data provider,
+ * such as "gdal" or "wms", and a url defining the specific data set to connect to.
+ * The raster layer constructor in turn instantiates a QgsRasterDataProvider subclass
+ * corresponding to the provider type, and passes it the url. The data provider
+ * connects to the data source.
  *
  *  Sample usage of the QgsRasterLayer class:
  *
- * \code{.cpp}
- *     QString myFileNameQString = "/path/to/file";
- *     QString myBaseNameQString = "my layer";
- *     QgsRasterLayer *myRasterLayer = new QgsRasterLayer(myFileNameQString, myBaseNameQString);
+ * \code{.py}
+ *     my_raster_layer = QgsRasterLayer("/path/to/file.tif", "my layer")
  * \endcode
- *
- *  In order to automate redrawing of a raster layer, you should link it to a map canvas like this :
- *
- * \code{.cpp}
- *     QObject::connect( myRasterLayer, SIGNAL(repaintRequested()), mapCanvas, SLOT(refresh()) );
- * \endcode
- *
- * Once a layer has been created you can find out what type of layer it is (GrayOrUndefined, Palette or Multiband):
- *
- * \code{.cpp}
- *    if (rasterLayer->rasterType()==QgsRasterLayer::Multiband)
- *    {
- *      //do something
- *    }
- *    else if (rasterLayer->rasterType()==QgsRasterLayer::Palette)
- *    {
- *      //do something
- *    }
- *    else // QgsRasterLayer::GrayOrUndefined
- *    {
- *      //do something.
- *    }
- * \endcode
- *
- *  Raster layers can also have an arbitrary level of transparency defined, and have their
- *  color palettes inverted using the setTransparency and setInvertHistogram methods.
- *
- *  Pseudocolor images can have their output adjusted to a given number of standard
- *  deviations using the setStandardDeviations method.
- *
- *  The final area of functionality you may be interested in is band mapping. Band mapping
- *  allows you to choose arbitrary band -> color mappings and is applicable only to Palette
- *  and Multiband rasters, There are four mappings that can be made: red, green, blue and gray.
- *  Mappings are non-exclusive. That is a given band can be assigned to no, some or all
- *  color mappings. The constructor sets sensible defaults for band mappings but these can be
- *  overridden at run time using the setRedBandName, setGreenBandName, setBlueBandName and setGrayBandName
- *  methods.
  */
-
 class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
 {
     Q_OBJECT
@@ -189,6 +121,21 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
        */
       QgsCoordinateTransformContext transformContext = QgsCoordinateTransformContext();
 
+      /**
+       * Controls whether the layer is allowed to have an invalid/unknown CRS.
+       *
+       * If TRUE, then no validation will be performed on the layer's CRS and the layer
+       * layer's crs() may be invalid() (i.e. the layer will have no georeferencing available
+       * and will be treated as having purely numerical coordinates).
+       *
+       * If FALSE (the default), the layer's CRS will be validated using QgsCoordinateReferenceSystem::validate(),
+       * which may cause a blocking, user-facing dialog asking users to manually select the correct CRS for the
+       * layer.
+       *
+       * \since QGIS 3.10
+       */
+      bool skipCrsValidation = false;
+
     };
 
     /**
@@ -196,16 +143,13 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      *
      * The main tasks carried out by the constructor are:
      *
-     * -Load the rasters default style (.qml) file if it exists
+     * - Load the rasters default style (.qml) file if it exists
+     * - Populate the RasterStatsVector with initial values for each band.
+     * - Calculate the layer extents
+     * - Determine whether the layer is gray, paletted or multiband.
+     * - Assign sensible defaults for the red, green, blue and gray bands.
      *
-     * -Populate the RasterStatsVector with initial values for each band.
-     *
-     * -Calculate the layer extents
-     *
-     * -Determine whether the layer is gray, paletted or multiband.
-     *
-     * -Assign sensible defaults for the red, green, blue and gray bands.
-     * */
+     */
     explicit QgsRasterLayer( const QString &uri,
                              const QString &baseName = QString(),
                              const QString &providerType = "gdal",
@@ -264,9 +208,10 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      * Set the data provider.
      * \param provider provider key string, must match a valid QgsRasterDataProvider key. E.g. "gdal", "wms", etc.
      * \param options provider options
+     * \param flags provider flags since QGIS 3.16
      * \since QGIS 3.2
      */
-    void setDataProvider( const QString &provider, const QgsDataProvider::ProviderOptions &options );
+    void setDataProvider( const QString &provider, const QgsDataProvider::ProviderOptions &options, QgsDataProvider::ReadFlags flags = QgsDataProvider::ReadFlags() );
 
     /**
      * Updates the data source of the layer. The layer's renderer and legend will be preserved only
@@ -287,16 +232,61 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      */
     LayerType rasterType() { return mRasterType; }
 
-    //! Sets raster renderer. Takes ownership of the renderer object
+    /**
+     * Sets the raster's \a renderer. Takes ownership of the renderer object.
+     * \see renderer()
+     */
     void setRenderer( QgsRasterRenderer *renderer SIP_TRANSFER );
+
+    /**
+     * Returns the raster's renderer.
+     *
+     * \see setRenderer()
+     */
     QgsRasterRenderer *renderer() const { return mPipe.renderer(); }
 
-    //! Sets raster resample filter. Takes ownership of the resample filter object
+    /**
+     * Returns the raster's resample filter.
+     *
+     * \see brightnessFilter()
+     * \see hueSaturationFilter()
+     */
     QgsRasterResampleFilter *resampleFilter() const { return mPipe.resampleFilter(); }
 
+    /**
+     * Returns the raster's brightness/contrast filter.
+     *
+     * \see resampleFilter()
+     * \see hueSaturationFilter()
+     */
     QgsBrightnessContrastFilter *brightnessFilter() const { return mPipe.brightnessFilter(); }
+
+    /**
+     * Returns the raster's hue/saturation filter.
+     *
+     * \see resampleFilter()
+     * \see brightnessFilter()
+     */
     QgsHueSaturationFilter *hueSaturationFilter() const { return mPipe.hueSaturationFilter(); }
 
+    /**
+     * Select which stage of the pipe should apply resampling.
+     *
+     * \see QgsRasterPipe::setResamplingStage()
+     *
+     * \since QGIS 3.16
+     */
+    void setResamplingStage( QgsRasterPipe::ResamplingStage stage );
+
+    /**
+     * Returns which stage of the pipe should apply resampling.
+     *
+     * \see QgsRasterPipe::resamplingStage()
+     *
+     * \since QGIS 3.16
+     */
+    QgsRasterPipe::ResamplingStage resamplingStage() const { return mPipe.resamplingStage(); }
+
     /**
      * Returns the raster pipe.
      */
@@ -324,10 +314,18 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      */
     QString bandName( int bandNoInt ) const;
 
-    // Returns nullptr if not using the data provider model (i.e. directly using GDAL)
+    /**
+     * Returns the source data provider.
+     *
+     * This will be NULLPTR if the layer is invalid.
+     */
     QgsRasterDataProvider *dataProvider() override;
 
-    // Returns nullptr if not using the data provider model (i.e. directly using GDAL)
+    /**
+     * Returns the source data provider.
+     *
+     * This will be NULLPTR if the layer is invalid.
+     */
     const QgsRasterDataProvider *dataProvider() const SIP_PYNAME( constDataProvider ) override;
 
     void reload() override;
@@ -379,9 +377,8 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      *  \param limits Limits
      *  \param extent Extent used to calculate limits, if empty, use full layer extent
      *  \param sampleSize Size of data sample to calculate limits, if 0, use full resolution
-     *  \param generateLookupTableFlag Generate lookup table. */
-
-
+     *  \param generateLookupTableFlag Generate lookup table.
+    */
     void setContrastEnhancement( QgsContrastEnhancement::ContrastEnhancementAlgorithm algorithm,
                                  QgsRasterMinMaxOrigin::Limits limits = QgsRasterMinMaxOrigin::MinMax,
                                  const QgsRectangle &extent = QgsRectangle(),
@@ -400,6 +397,23 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      */
     void refreshRendererIfNeeded( QgsRasterRenderer *rasterRenderer, const QgsRectangle &extent ) SIP_SKIP;
 
+    /**
+     * Returns the string (typically sql) used to define a subset of the layer.
+     * \returns The subset string or null QString if not implemented by the provider
+     * \since QGIS 3.12
+     */
+    virtual QString subsetString() const;
+
+    /**
+     * Sets the string (typically sql) used to define a subset of the layer
+     * \param subset The subset string. This may be the where clause of a sql statement
+     *               or other definition string specific to the underlying dataprovider
+     *               and data store.
+     * \returns TRUE, when setting the subset string was successful, FALSE otherwise
+     * \since QGIS 3.12
+     */
+    virtual bool setSubsetString( const QString &subset );
+
     /**
      * Returns default contrast enhancement settings for that type of raster.
      *  \note not available in Python bindings
@@ -415,13 +429,15 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
 
     /**
      * \brief Draws a preview of the rasterlayer into a QImage
-     \since QGIS 2.4 */
+     * \since QGIS 2.4
+    */
     QImage previewAsImage( QSize size, const QColor &bgColor = Qt::white,
                            QImage::Format format = QImage::Format_ARGB32_Premultiplied );
 
     void setLayerOrder( const QStringList &layers ) override;
     void setSubLayerVisibility( const QString &name, bool vis ) override;
     QDateTime timestamp() const override;
+    bool accept( QgsStyleEntityVisitorInterface *visitor ) const override;
 
     /**
      * Writes the symbology of the layer into the document provided in SLD 1.0.0 format
@@ -434,6 +450,17 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      */
     bool writeSld( QDomNode &node, QDomDocument &doc, QString &errorMessage, const QgsStringMap &props = QgsStringMap() ) const;
 
+    /**
+     * If the ignoreExtent flag is set, the layer will also render outside the
+     * bounding box reported by the data provider.
+     * To be used for example for WMS layers with labels or symbology that happens
+     * to be drawn outside the data extent.
+     *
+     * \since QGIS 3.10
+     */
+    bool ignoreExtents() const;
+
+    QgsMapLayerTemporalProperties *temporalProperties() override;
 
   public slots:
     void showStatusMessage( const QString &message );
@@ -445,6 +472,15 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      */
     virtual void setTransformContext( const QgsCoordinateTransformContext &transformContext ) override;
 
+  signals:
+
+    /**
+     * Emitted when the layer's subset string has changed.
+     * \since QGIS 3.12
+     */
+    void subsetStringChanged();
+
+
   protected:
     bool readSymbology( const QDomNode &node, QString &errorMessage, QgsReadWriteContext &context, QgsMapLayer::StyleCategories categories = QgsMapLayer::AllStyleCategories ) override;
     bool readStyle( const QDomNode &node, QString &errorMessage, QgsReadWriteContext &context, QgsMapLayer::StyleCategories categories = QgsMapLayer::AllStyleCategories ) override;
@@ -456,6 +492,7 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     bool writeXml( QDomNode &layer_node, QDomDocument &doc, const QgsReadWriteContext &context ) const override;
     QString encodedSource( const QString &source, const QgsReadWriteContext &context ) const override;
     QString decodedSource( const QString &source, const QString &provider,  const QgsReadWriteContext &context ) const override;
+
   private:
     //! \brief Initialize default values
     void init();
@@ -476,6 +513,9 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
                                  bool generateLookupTableFlag,
                                  QgsRasterRenderer *rasterRenderer );
 
+    //! Refresh renderer
+    void refreshRenderer( QgsRasterRenderer *rasterRenderer, const QgsRectangle &extent );
+
     void computeMinMax( int band,
                         const QgsRasterMinMaxOrigin &mmo,
                         QgsRasterMinMaxOrigin::Limits limits,
@@ -490,6 +530,9 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     //! Pointer to data provider
     QgsRasterDataProvider *mDataProvider = nullptr;
 
+    //! Pointer to temporal properties
+    QgsRasterLayerTemporalProperties *mTemporalProperties = nullptr;
+
     //! [ data provider interface ] Timestamp, the last modified time of the data source when the layer was created
     QDateTime mLastModified;
 
@@ -501,6 +544,9 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
 
     //! To save computations and possible infinite cycle of notifications
     QgsRectangle mLastRectangleUsedByRefreshContrastEnhancementIfNeeded;
+
+    QDomDocument mOriginalStyleDocument;
+    QDomElement mOriginalStyleElement;
 };
 
 // clazy:excludeall=qstring-allocations
